# -*- coding: utf-8 -*-

"""
Math extension for Python-Markdown
==================================

Adds support for displaying math formulas using MathJax / KaTeX

Copyright 2019-2020 Gautam Iyer <gi1242+mdxmath@gmail.com>

Based on code originally written by Dmitry Shachnev <mitya57@gmail.com>
"""

from markdown.inlinepatterns import InlineProcessor
from markdown.extensions import Extension


class inlineMathProcessor(InlineProcessor):
    def handleMatch(self, m, data):
        # MathJAX handles all the math. Just set the uses_math flag, and
        # protect the contents from markdown expansion.
        self.md.uses_math = True
        return m.group(0), m.start(0), m.end(0)


class MathExtension(Extension):
    def __init__(self, *args, **kwargs):
        self.config = {
            "enable_dollar_delimiter": [False, "Enable single-dollar delimiter"],
        }
        super(MathExtension, self).__init__(*args, **kwargs)

    def extendMarkdown(self, md):
        md.registerExtension(self)

        mathRegExps = [
            r"(?<!\\)\\\((.+?)\\\)",  # \( ... \)
            r"(?<!\\)\$\$.+?\$\$",  # $$ ... $$
            r"(?<!\\)\\\[.+?\\\]",  # \[ ... \]
            r"(?<!\\)\\begin{([a-z]+?\*?)}.+?\\end{\1}",
        ]
        if self.getConfig("enable_dollar_delimiter"):
            md.ESCAPED_CHARS.append("$")
            mathRegExps.append(r"(?<!\\|\$)\$.+?\$")  # $ ... $
        for i, pattern in enumerate(mathRegExps):
            # we should have higher priority than 'escape' which has 180
            md.inlinePatterns.register(
                inlineMathProcessor(pattern, md), f"math-inline-{i}", 185
            )

        md.uses_math = False
        self.md = md

    def reset(self):
        self.md.uses_math = False


def makeExtension(*args, **kwargs):
    return MathExtension(*args, **kwargs)



import argparse
from jyyslide_md.converter import converter


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("filepath", help="select a Markdown file to convert", type=str)
    parser.add_argument(
        "--watch",
        help="watch the file for changes, requires watchdog library",
        action=argparse.BooleanOptionalAction,
    )
    args = parser.parse_args()
    converter(args.filepath)

    if args.watch:
        print("watching file for changes")
        import livereload

        def generate_slides():
            converter(args.filepath)

        server = livereload.Server()
        server.watch(args.filepath, generate_slides)
        server.serve()


if __name__ == "__main__":
    main()



import os, shutil, uuid
from typing import Optional, Tuple
from .str_util import is_url


def get_files_under_folder(
    folerpath: str, suffix_name: Optional[str] = None
) -> list[str]:
    """返回目录folderpath下后缀名为suffix_name的所有文件的绝对路径列表"""
    return [
        os.path.abspath(os.path.join(dirpath, filename))
        for dirpath, dirnames, filenames in os.walk(folerpath)
        for filename in filenames
        if suffix_name is None or str(filename).endswith("." + suffix_name)
    ]


def read(filepath: str) -> str:  # 读取文本文件内容
    if os.path.exists(filepath) is False:
        raise Exception("The path {} is not exists".format(filepath))
    with open(filepath, "r", encoding="utf-8") as f:
        return str(f.read())


def write(filepath: str, data: str) -> None:  # 向文件(覆)写内容
    dirpath = os.path.dirname(filepath)
    if os.path.exists(dirpath) is False:
        os.makedirs(dirpath)
    with open(filepath, "w", encoding="utf-8") as f:
        f.write(data)


def get_abspath(basefile: str, filepath: str) -> str:
    return os.path.normpath(os.path.join(os.path.dirname(basefile), filepath))


def get_image_to_target(
    link: str, from_filepath: str, target_foldpath: str
) -> Tuple[str, bool]:
    # 对于from_filepath(请使用其绝对地址)中的图床链接link, 它可能是url、绝对地址或相对地址, 我们会get它然后重命名并放到target_foldpath下, 并返回重命名后的名字
    # 这里对图片类型的判断是通过link的后缀名, 有些图片的url的末尾不是类型名, 就会有bug
    name = uuid.uuid4().hex + "." + link.split(".")[-1]
    if is_url(link):
        pass
    else:
        if os.path.isabs(link) is True:
            pass
        else:
            link = get_abspath(from_filepath, link)
            pass

    if is_url(link):
        raise NotImplementedError("The url is not supported: ", link)
    else:
        if os.path.exists(link) is False:
            print("The path is not exists: ", link)
            return "", True
        shutil.copyfile(link, os.path.join(target_foldpath, name))

    return name, False



import re
from typing import List, Union


def process_images(content, func):
    """处理Markdown类型字符串中的图片链接, 返回处理过图片链接部分的Markdown字符串

    Args:
        content (_type_): Markdown类型字符串
        func (_type_): 处理图片链接的函数, 该函数接受图片链接字符串, 返回一个(有关图片链接的新串, 是否有错误)的元组
    """

    def modify(match):
        # 下面是黑盒魔法
        tar = match.group()
        pre, mid, suf = str(), str(), str()
        if tar[-1] == ")":
            pre = tar[: tar.index("(") + 1]
            mid = tar[tar.index("(") + 1 : -1]
            suf = tar[-1]
        else:
            mid = re.search(r'src="([^"]*)"', tar).group(1)
            pre, suf = tar.split(mid)

        link = mid
        # 黑盒魔法结束
        new_name, err = func(link)
        return pre + (new_name if err is False else link) + suf

    patten = r"!\[.*?\]\((.*?)\)|<img.*?src=[\'\"](.*?)[\'\"].*?>"
    return re.sub(patten, modify, content)


###

from markdown import markdown
from markdown import Extension
from markdown.blockprocessors import BlockProcessor
from .mdx_math import MathExtension
import xml.etree.ElementTree as etree


def md_to_html(md: str) -> str:
    class BoxBlockProcessor(BlockProcessor):
        first = True

        def run(self, parent, blocks):
            if self.first:
                self.first = False
                e = etree.SubElement(parent, "div")
                self.parser.parseBlocks(e, blocks)
                for _ in range(0, len(blocks)):
                    blocks.pop(0)
                return True
            return False

    class BoxExtension(Extension):
        def extendMarkdown(self, md):
            md.parser.blockprocessors.register(BoxBlockProcessor(md.parser), "box", 175)

    extensions: List[Union[str, BoxExtension]] = [
        BoxExtension(),
        MathExtension(),
        "meta",
        "fenced_code",
        "codehilite",
        "extra",
        "attr_list",
        "tables",
        "toc",
    ]
    return markdown(md, extensions=extensions)



import os

from jinja2 import Template

projects_abspath = os.path.split(os.path.realpath(__file__))[0]
static_path = os.path.join(projects_abspath, "static")
template_from = os.path.join(static_path, "template", "basetemp.html")
authortemp_from = os.path.join(static_path, "template", "authortemp.html")

op_first_section = "\n---\n"
op_second_section = "\n----\n"
op_animate_section = "\n++++\n"
op_index_fragment = "\n--\n"
op_front_matter = "\n+++++\n"

# 文件信息
filename = str()
filepath = str()
output_foldname = "dist"
output_filename = "index.html"
output_foldpath = str()
output_filepath = str()
static_foldpath = str()
images_foldname = (
    "img"  # 如果要修改还要修炼static下的img文件名，template中icon的文件路径
)
images_foldpath = str()

content = str()  # MD内容

template: str | Template = str()  # HTML内容
title = str()
body = str()

# 作者信息相关
author_template: str | Template = str()


def Init(target_filepath):
    global filename, filepath, output_foldname, output_filename, output_foldpath, output_filepath, static_foldpath, images_foldpath
    filename = os.path.basename(target_filepath)
    filepath = os.path.abspath(target_filepath)
    output_foldpath = os.path.join(filepath.split(filename)[0], output_foldname)
    output_filepath = os.path.join(output_foldpath, output_filename)

    static_foldpath = os.path.join(output_foldpath, "static")
    images_foldpath = os.path.join(static_foldpath, images_foldname)



import re
import json
import os
import shutil

import yaml
from jinja2 import Template
from pyquery import PyQuery  # type: ignore

from .util import *
from . import settings as st


def process_html_elements(before_html):
    temp_html = "<html><body>{}</body></html>".format(before_html)
    page = PyQuery(temp_html)
    e = page
    for item in e("h1").parent():
        t = PyQuery(item)
        t.wrap("<div style='width:100%'>")
        t.wrap("<div class='center middle'>")
    class_data = {
        "ul": "list-disc font-serif",
        "li": "ml-8",
        "h2": "text-xl mt-2 pb-2 font-sans",
        "h1": "text-2xl mt-2 font-sans",
        "p": "font-serif my-1",
        "pre": "bg-gray-100 overflow-x-auto rounded p-2 mb-2 mt-2",
        # "img": "center",  # 设置图片是否默认居中
    }
    for k, v in class_data.items():
        for item in e(k):
            t = PyQuery(item)
            t.add_class(v)
    page = e
    items = page("body").children()
    return "".join([str(PyQuery(e)) for e in items])


def process_terminal(semi_html):
    semi_html += st.author_template
    st.author_template = ""
    temp = "<div>" + semi_html + "</div>"
    semi_html = process_html_elements(temp)
    return semi_html


def vertical_to_fragment(vertical: str) -> str:
    fragments = vertical.split(st.op_index_fragment)

    fragment_list = [md_util.md_to_html(fragments[0])]
    template = "<div class='fragment' data-fragment-index='{}'>{}</div>"

    for i in range(1, len(fragments)):
        fragment_list.append(template.format(i, md_util.md_to_html(fragments[i])))

    return "".join(fragment_list)


def vertical_to_animate(vertical: str) -> str:
    animates = vertical.split(st.op_animate_section)

    animate_list = []
    template = "{}"

    for i in range(len(animates)):
        animate_list.append(template.format(md_util.md_to_html(animates[i])))

    return "".join(animate_list)


def horizontal_to_vertical(horizontal: str) -> str:
    verticals_divided_by_second = horizontal.split(st.op_second_section)

    sections = []
    template_second = "<section>{}</section>"

    for vertical_divided_by_second in verticals_divided_by_second:
        if vertical_divided_by_second.isspace():
            continue
        if st.op_animate_section in vertical_divided_by_second:
            verticals_divided_by_animate = vertical_divided_by_second.split(
                st.op_animate_section
            )
            template_animate = "<section data-auto-animate>{}</section>"
            for vertical_divided_by_animate in verticals_divided_by_animate:
                if vertical_divided_by_animate.isspace():
                    continue
                sections.append(
                    template_animate.format(
                        process_terminal(
                            vertical_to_animate(vertical_divided_by_animate)
                        )
                    )
                )
        elif st.op_index_fragment in vertical_divided_by_second:
            sections.append(
                template_second.format(
                    process_terminal(vertical_to_fragment(vertical_divided_by_second))
                )
            )
        else:
            sections.append(
                template_second.format(
                    process_terminal(md_util.md_to_html(vertical_divided_by_second))
                )
            )

    return "".join(sections)


def md_divide_to_horizontal(content: str):
    horizontals = content.split(st.op_first_section)

    sections = []
    template = "<section>{}</section>"

    for horizontal in horizontals:
        if horizontal.isspace():
            continue
        html_second_sections = horizontal_to_vertical(horizontal)

        html = template.format(html_second_sections)
        sections.append(html)

    return "".join(sections)


MULCOLs = '<div class="mul-cols">'
COLS = '<div class="col">'
END = "</div>"

DIRTY_MULCOLS = "\nJYYMULCOLS\n"
DIRTY_COLS = "\nJYYCOL\n"
DIRTY_END = "\nJYYEND\n"

DIRTY2_MULCOLS = '<p class="font-serif my-1">JYYMULCOLS</p>'
DIRTY2_COLS = '<p class="font-serif my-1">JYYCOL</p>'
DIRTY2_END = '<p class="font-serif my-1">JYYEND</p>'

splitter = re.compile(rf"{MULCOLs}|{COLS}|{END}")


def get_body(content):
    content = content.replace(MULCOLs, DIRTY_MULCOLS)
    content = content.replace(COLS, DIRTY_COLS)
    content = content.replace(END, DIRTY_END)

    content = md_divide_to_horizontal(content)

    content = content.replace(DIRTY2_MULCOLS, MULCOLs)
    content = content.replace(DIRTY2_COLS, COLS)
    content = content.replace(DIRTY2_END, END)

    return content


def process_image_link():
    def func(link):
        new_name, err = file_util.get_image_to_target(
            link, st.filepath, st.images_foldpath
        )

        return (
            os.path.join(".", "static", st.images_foldname, new_name)
            if err is False
            else ""
        ), err

    st.content = md_util.process_images(st.content, func)


def process_front_matter():
    if st.op_front_matter not in st.content:
        st.author_template = ""
        return

    parts = st.content.split(st.op_front_matter)

    front_matter = parts[0]
    st.content = "".join(parts[1:])

    try:
        data = json.loads(front_matter)
    except Exception as e:
        data = yaml.load(front_matter, Loader=yaml.SafeLoader)

    if "departments" not in data:
        data["departments"] = []

    for department in data["departments"]:
        new_name, err = file_util.get_image_to_target(
            department["img"], st.filepath, st.images_foldpath
        )
        if err is False:
            department["img"] = os.path.join(
                ".", "static", st.images_foldname, new_name
            )
        department["name"] = department["name"].replace(" ", "&#12288;")

    st.author_template = st.author_template.render(author=data["author"], departments=data["departments"])  # type: ignore


def process_static():
    if os.path.exists(st.output_foldpath) is True:
        shutil.rmtree(st.output_foldpath)
    os.mkdir(st.output_foldpath)
    shutil.copytree(st.static_path, st.static_foldpath)


def converter(MDfilepath):
    st.Init(MDfilepath)
    process_static()

    st.template = Template(file_util.read(st.template_from))
    st.author_template = Template(file_util.read(st.authortemp_from))

    st.content = file_util.read(st.filepath)
    process_front_matter()  # 解析markdown中的front_matter and render author_template(will change content)
    process_image_link()  # 处理Markdown文本中的图片链接, 将它们get到静态文件, 同时修改content

    st.title = "".join(st.filename.split(".")[:-1])
    st.body = get_body(st.content)

    st.template = st.template.render(title=st.title, body=st.body)

    file_util.write(st.output_filepath, st.template)



from urllib.parse import urlparse


# 分辨URL和路径: 判断一个字符串是否为URL
def is_url(string):
    result = urlparse(string)
    return all([result.scheme, result.netloc])


# 分辨URL和路径: 判断一个字符串是否为路径
def is_path(string):
    result = urlparse(string)
    return not all([result.scheme, result.netloc])
